package com.outr.webcommunicator.netty.communicator

import com.outr.webcommunicator._
import com.outr.webcommunicator.netty.handler.WebSocketHandler
import org.jboss.netty.channel.ChannelHandlerContext
import org.jboss.netty.handler.codec.http.websocketx.{BinaryWebSocketFrame, TextWebSocketFrame}
import annotation.tailrec
import com.outr.webcommunicator.Event

import org.powerscala.json._

/**
 * @author Matt Hicks <matt@outr.com>
 */
class CommunicatorWebSocketHandler[C <: Communicator](val manager: NettyCommunicatorManager[C])
  extends WebSocketHandler with CommunicatorReceiver[C] with CommunicatorSender[C] {

  def handleText(context: ChannelHandlerContext, frame: TextWebSocketFrame) = {
    val event = parse[Event](frame.getText)
    handle(event)
  }

  def handleBinary(context: ChannelHandlerContext, frame: BinaryWebSocketFrame) = {
    println("*** handleBinary currently not supported!")
  }

  override def communicatorChanged() {
    super.communicatorChanged()

    communicator.communicatorSender = this
    outgoingEnqueued()    // Send the queue
  }

  override def outgoingEnqueued() {
    super.outgoingEnqueued()

    sendQueue(communicator.outgoing(lastSentId))
  }

  override def communicatorUpdating() {
    super.communicatorUpdating()

    if (context != null && context.getChannel.isOpen) {
      communicator.updated()    // Always keep this updated so WebSocket never times out
    } else {
      if (communicator.communicatorReceiver == this) {
        communicator.communicatorReceiver = null
      }
      if (communicator.communicatorSender == this) {
        communicator.communicatorSender = null
      }
    }
  }

  @tailrec
  final def sendQueue(messages: Seq[IdentifiableMessage]): Unit = {
    if (messages.nonEmpty) {
      val head = messages.head
      send(generate(head))
      lastSentId = head.id
      sendQueue(messages.tail)
    }
  }
}
