package com.outr.webcommunicator.proxy

import org.jboss.netty.channel._
import org.jboss.netty.buffer.ChannelBuffer

/**
 * @author Matt Hicks <mhicks@outr.com>
 */
class OutboundHandler(handler: ProxyHandler) extends SimpleChannelUpstreamHandler {
  override def messageReceived(ctx: ChannelHandlerContext, e: MessageEvent) {
    val msg = e.getMessage.asInstanceOf[ChannelBuffer]
    handler.synchronized {
      handler.inboundChannel.write(msg)
      // If inboundChannel is saturated, do not read until notified in ProxyHandler.channelInterestChanged()
      if (!handler.inboundChannel.isWritable) {
        e.getChannel.setReadable(false)
      }
    }
  }

  override def channelInterestChanged(ctx: ChannelHandlerContext, e: ChannelStateEvent) {
    // If outboundChannel is not saturated anymore, continue accepting
    // the incoming traffic from the inboundChannel.
    handler.synchronized {
      if (e.getChannel.isWritable) {
        handler.inboundChannel.setReadable(true)
      }
    }
  }

  override def channelClosed(ctx: ChannelHandlerContext, e: ChannelStateEvent) {
    ProxyHandler.closeOnFlush(handler.inboundChannel)
  }

  override def exceptionCaught(ctx: ChannelHandlerContext, e: ExceptionEvent) {
    e.getCause.printStackTrace()
    ProxyHandler.closeOnFlush(handler.inboundChannel)
  }
}
